@@ -2,7 +2,7 @@
  *   move.c  --  This file is part of GNU nano.                           *
  *                                                                        *
  *   Copyright (C) 1999-2011, 2013-2017 Free Software Foundation, Inc.    *
- *   Copyright (C) 2014, 2015, 2016 Benno Schulenberg                     *
+ *   Copyright (C) 2014-2017 Benno Schulenberg                            *
  *                                                                        *
  *   GNU nano is free software: you can redistribute it and/or modify     *
  *   it under the terms of the GNU General Public License as published    *
@@ -22,7 +22,6 @@
 #include "proto.h"
 
 #include <string.h>
-#include <ctype.h>
 
 /* Move to the first line of the file. */
 void do_first_line(void)
@@ -53,13 +52,10 @@ void get_edge_and_target(size_t *leftedge, size_t *target_column)
 {
 #ifndef NANO_TINY
     if (ISSET(SOFTWRAP)) {
-	size_t realspan = strlenpt(openfile->current->data);
+	size_t shim = editwincols * (1 + (tabsize / editwincols));
 
-	if (realspan > openfile->placewewant)
-	    realspan = openfile->placewewant;
-
-	*leftedge = (realspan / editwincols) * editwincols;
-	*target_column = openfile->placewewant % editwincols;
+	*leftedge = leftedge_for(xplustabs(), openfile->current);
+	*target_column = (openfile->placewewant + shim - *leftedge) % editwincols;
     } else
 #endif
     {
@@ -68,6 +64,51 @@ void get_edge_and_target(size_t *leftedge, size_t *target_column)
     }
 }
 
+/* Return the index in line->data that corresponds to the given column on the
+ * chunk that starts at the given leftedge.  If the target column has landed
+ * on a tab, prevent the cursor from falling back a row when moving forward,
+ * or from skipping a row when moving backward, by incrementing the index. */
+size_t proper_x(filestruct *line, size_t *leftedge, bool forward,
+		size_t column, bool *shifted)
+{
+    size_t index = actual_x(line->data, column);
+
+#ifndef NANO_TINY
+    if (ISSET(SOFTWRAP) && line->data[index] == '\t' &&
+		((forward && strnlenpt(line->data, index) < *leftedge) ||
+		(!forward && column / tabsize == (*leftedge - 1) / tabsize &&
+		column / tabsize < (*leftedge + editwincols - 1) / tabsize))) {
+	index++;
+
+	if (shifted != NULL)
+	    *shifted = TRUE;
+    }
+
+    if (ISSET(SOFTWRAP))
+	*leftedge = leftedge_for(strnlenpt(line->data, index), line);
+#endif
+
+    return index;
+}
+
+/* Adjust the values for current_x and placewewant in case we have landed in
+ * the middle of a tab that crosses a row boundary. */
+void set_proper_index_and_pww(size_t *leftedge, size_t target, bool forward)
+{
+    bool shifted = FALSE;
+    size_t was_edge = *leftedge;
+
+    openfile->current_x = proper_x(openfile->current, leftedge, forward,
+			actual_last_column(*leftedge, target), &shifted);
+
+    /* If the index was incremented, try going to the target column. */
+    if (shifted || *leftedge < was_edge)
+	openfile->current_x = proper_x(openfile->current, leftedge, forward,
+			actual_last_column(*leftedge, target), &shifted);
+
+    openfile->placewewant = *leftedge + target;
+}
+
 /* Move up nearly one screenful. */
 void do_page_up(void)
 {
@@ -78,11 +119,11 @@ void do_page_up(void)
      * beginning of the top line of the edit window, as Pico does. */
     if (!ISSET(SMOOTH_SCROLL)) {
 	openfile->current = openfile->edittop;
-	openfile->placewewant = openfile->firstcolumn;
+	leftedge = openfile->firstcolumn;
 	openfile->current_y = 0;
-    }
-
-    get_edge_and_target(&leftedge, &target_column);
+	target_column = 0;
+    } else
+	get_edge_and_target(&leftedge, &target_column);
 
     /* Move up the required number of lines or chunks.  If we can't, we're
      * at the top of the file, so put the cursor there and get out. */
@@ -91,9 +132,7 @@ void do_page_up(void)
 	return;
     }
 
-    openfile->placewewant = leftedge + target_column;
-    openfile->current_x = actual_x(openfile->current->data,
-					openfile->placewewant);
+    set_proper_index_and_pww(&leftedge, target_column, FALSE);
 
     /* Move the viewport so that the cursor stays immobile, if possible. */
     adjust_viewport(STATIONARY);
@@ -110,11 +149,11 @@ void do_page_down(void)
      * beginning of the top line of the edit window, as Pico does. */
     if (!ISSET(SMOOTH_SCROLL)) {
 	openfile->current = openfile->edittop;
-	openfile->placewewant = openfile->firstcolumn;
+	leftedge = openfile->firstcolumn;
 	openfile->current_y = 0;
-    }
-
-    get_edge_and_target(&leftedge, &target_column);
+	target_column = 0;
+    } else
+	get_edge_and_target(&leftedge, &target_column);
 
     /* Move down the required number of lines or chunks.  If we can't, we're
      * at the bottom of the file, so put the cursor there and get out. */
@@ -123,38 +162,30 @@ void do_page_down(void)
 	return;
     }
 
-    openfile->placewewant = leftedge + target_column;
-    openfile->current_x = actual_x(openfile->current->data,
-					openfile->placewewant);
+    set_proper_index_and_pww(&leftedge, target_column, TRUE);
 
     /* Move the viewport so that the cursor stays immobile, if possible. */
     adjust_viewport(STATIONARY);
     refresh_needed = TRUE;
 }
 
-#ifndef DISABLE_JUSTIFY
+#ifdef ENABLE_JUSTIFY
 /* Move to the beginning of the last beginning-of-paragraph line before the
  * current line.  If update_screen is TRUE, update the screen afterwards. */
 void do_para_begin(bool update_screen)
 {
     filestruct *was_current = openfile->current;
 
-    if (openfile->current != openfile->fileage) {
-	do
-	    openfile->current = openfile->current->prev;
-	while (!begpar(openfile->current));
-    }
+    if (openfile->current != openfile->fileage)
+	openfile->current = openfile->current->prev;
+
+    while (!begpar(openfile->current))
+	openfile->current = openfile->current->prev;
 
     openfile->current_x = 0;
 
     if (update_screen)
-	edit_redraw(was_current);
-}
-
-/* Move up to first start of a paragraph before the current line. */
-void do_para_begin_void(void)
-{
-    do_para_begin(TRUE);
+	edit_redraw(was_current, CENTERING);
 }
 
 /* Move down to the beginning of the last line of the current paragraph.
@@ -184,7 +215,13 @@ void do_para_end(bool update_screen)
 	openfile->current_x = strlen(openfile->current->data);
 
     if (update_screen)
-	edit_redraw(was_current);
+	edit_redraw(was_current, CENTERING);
+}
+
+/* Move up to first start of a paragraph before the current line. */
+void do_para_begin_void(void)
+{
+    do_para_begin(TRUE);
 }
 
 /* Move down to just after the first end of a paragraph. */
@@ -192,9 +229,9 @@ void do_para_end_void(void)
 {
     do_para_end(TRUE);
 }
-#endif /* !DISABLE_JUSTIFY */
+#endif /* ENABLE_JUSTIFY */
 
-/* Move to the preceding block of text in the file. */
+/* Move to the preceding block of text. */
 void do_prev_block(void)
 {
     filestruct *was_current = openfile->current;
@@ -213,10 +250,10 @@ void do_prev_block(void)
 	openfile->current = openfile->current->next;
 
     openfile->current_x = 0;
-    edit_redraw(was_current);
+    edit_redraw(was_current, CENTERING);
 }
 
-/* Move to the next block of text in the file. */
+/* Move to the next block of text. */
 void do_next_block(void)
 {
     filestruct *was_current = openfile->current;
@@ -231,12 +268,11 @@ void do_next_block(void)
     }
 
     openfile->current_x = 0;
-    edit_redraw(was_current);
+    edit_redraw(was_current, CENTERING);
 }
 
-/* Move to the previous word in the file.  If allow_punct is TRUE, treat
- * punctuation as part of a word.  If update_screen is TRUE, update the
- * screen afterwards. */
+/* Move to the previous word.  If allow_punct is TRUE, treat punctuation
+ * as part of a word.  When requested, update the screen afterwards. */
 void do_prev_word(bool allow_punct, bool update_screen)
 {
     filestruct *was_current = openfile->current;
@@ -274,23 +310,13 @@ void do_prev_word(bool allow_punct, bool update_screen)
 	openfile->current_x = move_mbright(openfile->current->data,
 						openfile->current_x);
 
-    if (update_screen) {
-	focusing = FALSE;
-	edit_redraw(was_current);
-    }
+    if (update_screen)
+	edit_redraw(was_current, FLOWING);
 }
 
-/* Move to the previous word in the file, treating punctuation as part of a
- * word if the WORD_BOUNDS flag is set, and update the screen afterwards. */
-void do_prev_word_void(void)
-{
-    do_prev_word(ISSET(WORD_BOUNDS), TRUE);
-}
-
-/* Move to the next word in the file.  If allow_punct is TRUE, treat
- * punctuation as part of a word.  If update_screen is TRUE, update the
- * screen afterwards.  Return TRUE if we started on a word, and FALSE
- * otherwise. */
+/* Move to the next word.  If allow_punct is TRUE, treat punctuation
+ * as part of a word.   When requested, update the screen afterwards.
+ * Return TRUE if we started on a word, and FALSE otherwise. */
 bool do_next_word(bool allow_punct, bool update_screen)
 {
     filestruct *was_current = openfile->current;
@@ -302,7 +328,7 @@ bool do_next_word(bool allow_punct, bool update_screen)
     while (TRUE) {
 	/* If at the end of a line, move to the beginning of the next one. */
 	if (openfile->current->data[openfile->current_x] == '\0') {
-	    /* If at the end of the file, stop. */
+	    /* When at end of file, stop. */
 	    if (openfile->current->next == NULL)
 		break;
 	    openfile->current = openfile->current->next;
@@ -323,15 +349,20 @@ bool do_next_word(bool allow_punct, bool update_screen)
 	    break;
     }
 
-    if (update_screen) {
-	focusing = FALSE;
-	edit_redraw(was_current);
-    }
+    if (update_screen)
+	edit_redraw(was_current, FLOWING);
 
     /* Return whether we started on a word. */
     return started_on_word;
 }
 
+/* Move to the previous word in the file, treating punctuation as part of a
+ * word if the WORD_BOUNDS flag is set, and update the screen afterwards. */
+void do_prev_word_void(void)
+{
+    do_prev_word(ISSET(WORD_BOUNDS), TRUE);
+}
+
 /* Move to the next word in the file, treating punctuation as part of a word
  * if the WORD_BOUNDS flag is set, and update the screen afterwards. */
 void do_next_word_void(void)
@@ -340,23 +371,24 @@ void do_next_word_void(void)
 }
 
 /* Move to the beginning of the current line (or softwrapped chunk).
- * If be_clever is TRUE, do a smart home when wanted and possible,
- * and do a dynamic home when in softwrap mode and it's possible.
- * If be_clever is FALSE, just do a simple home. */
-void do_home(bool be_clever)
+ * When enabled, do a smart home.  When softwrapping, go the beginning
+ * of the full line when already at the start of a chunk. */
+void do_home(void)
 {
     filestruct *was_current = openfile->current;
-    size_t was_column = openfile->placewewant;
+    size_t was_column = xplustabs();
     bool moved_off_chunk = TRUE;
 #ifndef NANO_TINY
     bool moved = FALSE;
-    size_t leftedge_x = 0;
+    size_t leftedge = 0, leftedge_x = 0;
 
-    if (ISSET(SOFTWRAP))
-	leftedge_x = actual_x(openfile->current->data,
-			(was_column / editwincols) * editwincols);
+    if (ISSET(SOFTWRAP)) {
+	leftedge = leftedge_for(was_column, openfile->current);
+	leftedge_x = proper_x(openfile->current, &leftedge, FALSE, leftedge,
+				NULL);
+    }
 
-    if (ISSET(SMART_HOME) && be_clever) {
+    if (ISSET(SMART_HOME)) {
 	size_t indent_x = indent_length(openfile->current->data);
 
 	if (openfile->current->data[indent_x] != '\0') {
@@ -376,11 +408,11 @@ void do_home(bool be_clever)
     if (!moved && ISSET(SOFTWRAP)) {
 	/* If already at the left edge of the screen, move fully home.
 	 * Otherwise, move to the left edge. */
-	if (was_column % editwincols == 0 && be_clever)
+	if (openfile->current_x == leftedge_x)
 	    openfile->current_x = 0;
 	else {
 	    openfile->current_x = leftedge_x;
-	    openfile->placewewant = (was_column / editwincols) * editwincols;
+	    openfile->placewewant = leftedge;
 	    moved_off_chunk = FALSE;
 	}
     } else if (!moved)
@@ -388,74 +420,72 @@ void do_home(bool be_clever)
 	openfile->current_x = 0;
 
     if (moved_off_chunk)
-	openfile->placewewant = 0;
+	openfile->placewewant = xplustabs();
 
     /* If we changed chunk, we might be offscreen.  Otherwise,
      * update current if the mark is on or we changed "page". */
-    if (ISSET(SOFTWRAP) && moved_off_chunk) {
-	focusing = FALSE;
-	edit_redraw(was_current);
-    } else if (line_needs_update(was_column, openfile->placewewant))
+    if (ISSET(SOFTWRAP) && moved_off_chunk)
+	edit_redraw(was_current, FLOWING);
+    else if (line_needs_update(was_column, openfile->placewewant))
 	update_line(openfile->current, openfile->current_x);
 }
 
-/* Do a (smart or dynamic) home. */
-void do_home_void(void)
-{
-    do_home(TRUE);
-}
-
 /* Move to the end of the current line (or softwrapped chunk).
- * If be_clever is TRUE, do a dynamic end when in softwrap mode and
- * it's possible.  If be_clever is FALSE, just do a simple end. */
-void do_end(bool be_clever)
+ * When softwrapping and alredy at the end of a chunk, go to the
+ * end of the full line. */
+void do_end(void)
 {
     filestruct *was_current = openfile->current;
-    size_t was_column = openfile->placewewant;
+    size_t was_column = xplustabs();
     size_t line_len = strlen(openfile->current->data);
     bool moved_off_chunk = TRUE;
 
 #ifndef NANO_TINY
     if (ISSET(SOFTWRAP)) {
-	size_t rightedge_x = actual_x(openfile->current->data,
-			((was_column / editwincols) * editwincols) +
-			(editwincols - 1));
+	bool last_chunk = FALSE;
+	size_t leftedge = leftedge_for(was_column, openfile->current);
+	size_t rightedge = get_softwrap_breakpoint(openfile->current->data,
+						leftedge, &last_chunk);
+	size_t rightedge_x;
+
+	/* If we're on the last chunk, we're already at the end of the line.
+	 * Otherwise, we're one column past the end of the line.  Shifting
+	 * backwards one column might put us in the middle of a multi-column
+	 * character, but actual_x() will fix that. */
+	if (!last_chunk)
+	    rightedge--;
+
+	rightedge_x = actual_x(openfile->current->data, rightedge);
 
 	/* If already at the right edge of the screen, move fully to
 	 * the end of the line.  Otherwise, move to the right edge. */
-	if (openfile->current_x == rightedge_x && be_clever)
+	if (openfile->current_x == rightedge_x)
 	    openfile->current_x = line_len;
 	else {
 	    openfile->current_x = rightedge_x;
+	    openfile->placewewant = rightedge;
 	    moved_off_chunk = FALSE;
 	}
     } else
 #endif
 	openfile->current_x = line_len;
 
-    openfile->placewewant = xplustabs();
+    if (moved_off_chunk)
+	openfile->placewewant = xplustabs();
 
     /* If we changed chunk, we might be offscreen.  Otherwise,
      * update current if the mark is on or we changed "page". */
-    if (ISSET(SOFTWRAP) && moved_off_chunk) {
-	focusing = FALSE;
-	edit_redraw(was_current);
-    } else if (line_needs_update(was_column, openfile->placewewant))
+    if (ISSET(SOFTWRAP) && moved_off_chunk)
+	edit_redraw(was_current, FLOWING);
+    else if (line_needs_update(was_column, openfile->placewewant))
 	update_line(openfile->current, openfile->current_x);
 }
 
-/* Do a (dynamic) end. */
-void do_end_void(void)
-{
-    do_end(TRUE);
-}
-
 /* Move the cursor to the preceding line or chunk.  If scroll_only is TRUE,
  * also scroll the screen one row, so the cursor stays in the same spot. */
 void do_up(bool scroll_only)
 {
     filestruct *was_current = openfile->current;
-    size_t was_column = xplustabs();
     size_t leftedge, target_column;
 
     /* When just scrolling and the top of the file is onscreen, get out. */
@@ -469,34 +499,15 @@ void do_up(bool scroll_only)
     if (go_back_chunks(1, &openfile->current, &leftedge) > 0)
 	return;
 
+    set_proper_index_and_pww(&leftedge, target_column, FALSE);
+
+    if (scroll_only)
+	edit_scroll(BACKWARD, 1);
+
+    edit_redraw(was_current, FLOWING);
+
+    /* <Up> should not change placewewant, so restore it. */
     openfile->placewewant = leftedge + target_column;
-    openfile->current_x = actual_x(openfile->current->data,
-					openfile->placewewant);
-
-    /* When the cursor was on the first line of the edit window (or when just
-     * scrolling without moving the cursor), scroll the edit window up -- one
-     * line if we're in smooth scrolling mode, and half a page otherwise. */
-    if (openfile->current_y == 0 || scroll_only)
-	edit_scroll(UPWARD, (ISSET(SMOOTH_SCROLL) || scroll_only) ?
-				1 : editwinrows / 2 + 1);
-
-    if (openfile->current_y > 0) {
-	/* Redraw the prior line if it's not offscreen, and it's not the same
-	 * line as the current one, and the line was horizontally scrolled. */
-	if ((!scroll_only || openfile->current_y < editwinrows - 1) &&
-			openfile->current != was_current &&
-			line_needs_update(was_column, 0))
-	    update_line(openfile->current->next, 0);
-	/* Redraw the current line if it needs to be horizontally scrolled. */
-	if (line_needs_update(0, xplustabs()))
-	    update_line(openfile->current, openfile->current_x);
-    }
-}
-
-/* Move up one line or chunk. */
-void do_up_void(void)
-{
-    do_up(FALSE);
 }
 
 /* Move the cursor to next line or chunk.  If scroll_only is TRUE, also
@@ -504,7 +515,6 @@ void do_up_void(void)
 void do_down(bool scroll_only)
 {
     filestruct *was_current = openfile->current;
-    size_t was_column = xplustabs();
     size_t leftedge, target_column;
 
     get_edge_and_target(&leftedge, &target_column);
@@ -513,28 +523,21 @@ void do_down(bool scroll_only)
     if (go_forward_chunks(1, &openfile->current, &leftedge) > 0)
 	return;
 
+    set_proper_index_and_pww(&leftedge, target_column, TRUE);
+
+    if (scroll_only)
+	edit_scroll(FORWARD, 1);
+
+    edit_redraw(was_current, FLOWING);
+
+    /* <Down> should not change placewewant, so restore it. */
     openfile->placewewant = leftedge + target_column;
-    openfile->current_x = actual_x(openfile->current->data,
-					openfile->placewewant);
+}
 
-    /* When the cursor was on the last line of the edit window (or when just
-     * scrolling without moving the cursor), scroll the edit window down -- one
-     * line if we're in smooth scrolling mode, and half a page otherwise. */
-    if (openfile->current_y == editwinrows - 1 || scroll_only)
-	edit_scroll(DOWNWARD, (ISSET(SMOOTH_SCROLL) || scroll_only) ?
-				1 : editwinrows / 2 + 1);
-
-    if (openfile->current_y < editwinrows - 1) {
-	/* Redraw the prior line if it's not offscreen, and it's not the same
-	 * line as the current one, and the line was horizontally scrolled. */
-	if ((!scroll_only || openfile->current_y > 0) &&
-			openfile->current != was_current &&
-			line_needs_update(was_column, 0))
-	    update_line(openfile->current->prev, 0);
-	/* Redraw the current line if it needs to be horizontally scrolled. */
-	if (line_needs_update(0, xplustabs()))
-	    update_line(openfile->current, openfile->current_x);
-    }
+/* Move up one line or chunk. */
+void do_up_void(void)
+{
+    do_up(FALSE);
 }
 
 /* Move down one line or chunk. */
@@ -560,71 +563,31 @@ void do_scroll_down(void)
 /* Move left one character. */
 void do_left(void)
 {
-    size_t was_column = xplustabs();
-#ifndef NANO_TINY
-    size_t was_chunk = (was_column / editwincols);
     filestruct *was_current = openfile->current;
-#endif
 
     if (openfile->current_x > 0)
 	openfile->current_x = move_mbleft(openfile->current->data,
 						openfile->current_x);
     else if (openfile->current != openfile->fileage) {
-	do_up_void();
-	do_end(FALSE);
-	return;
+	openfile->current = openfile->current->prev;
+	openfile->current_x = strlen(openfile->current->data);
     }
 
-    openfile->placewewant = xplustabs();
-
-#ifndef NANO_TINY
-    /* If we were on the first line of the edit window, and we changed chunk,
-     * we're now above the first line of the edit window, so scroll up. */
-    if (ISSET(SOFTWRAP) && openfile->current_y == 0 &&
-		openfile->current == was_current &&
-		openfile->placewewant / editwincols != was_chunk) {
-	edit_scroll(UPWARD, ISSET(SMOOTH_SCROLL) ? 1 : editwinrows / 2 + 1);
-	return;
-    }
-#endif
-
-    /* Update current if the mark is on or it has changed "page". */
-    if (line_needs_update(was_column, openfile->placewewant))
-	update_line(openfile->current, openfile->current_x);
+    edit_redraw(was_current, FLOWING);
 }
 
 /* Move right one character. */
 void do_right(void)
 {
-    size_t was_column = xplustabs();
-#ifndef NANO_TINY
-    size_t was_chunk = (was_column / editwincols);
     filestruct *was_current = openfile->current;
-#endif
 
     if (openfile->current->data[openfile->current_x] != '\0')
 	openfile->current_x = move_mbright(openfile->current->data,
 						openfile->current_x);
     else if (openfile->current != openfile->filebot) {
-	do_home(FALSE);
-	do_down_void();
-	return;
+	openfile->current = openfile->current->next;
+	openfile->current_x = 0;
     }
 
-    openfile->placewewant = xplustabs();
-
-#ifndef NANO_TINY
-    /* If we were on the last line of the edit window, and we changed chunk,
-     * we're now below the first line of the edit window, so scroll down. */
-    if (ISSET(SOFTWRAP) && openfile->current_y == editwinrows - 1 &&
-		openfile->current == was_current &&
-		openfile->placewewant / editwincols != was_chunk) {
-	edit_scroll(DOWNWARD, ISSET(SMOOTH_SCROLL) ? 1 : editwinrows / 2 + 1);
-	return;
-    }
-#endif
-
-    /* Update current if the mark is on or it has changed "page". */
-    if (line_needs_update(was_column, openfile->placewewant))
-	update_line(openfile->current, openfile->current_x);
+    edit_redraw(was_current, FLOWING);
 }
