@@ -1,9 +1,8 @@
 /**************************************************************************
  *   utils.c  --  This file is part of GNU nano.                          *
  *                                                                        *
- *   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007,  *
- *   2008, 2009, 2010, 2011, 2013, 2014 Free Software Foundation, Inc.    *
- *   Copyright (C) 2016 Benno Schulenberg                                 *
+ *   Copyright (C) 1999-2011, 2013-2017 Free Software Foundation, Inc.    *
+ *   Copyright (C) 2016-2017 Benno Schulenberg                            *
  *                                                                        *
  *   GNU nano is free software: you can redistribute it and/or modify     *
  *   it under the terms of the GNU General Public License as published    *
@@ -22,12 +21,12 @@
 
 #include "proto.h"
 
-#include <string.h>
-#include <stdio.h>
-#include <unistd.h>
-#include <pwd.h>
-#include <ctype.h>
 #include <errno.h>
+#ifdef HAVE_PWD_H
+#include <pwd.h>
+#endif
+#include <string.h>
+#include <unistd.h>
 
 /* Return the user's home directory.  We use $HOME, and if that fails,
  * we fall back on the home directory of the effective user ID. */
@@ -36,6 +35,7 @@ void get_homedir(void)
     if (homedir == NULL) {
 	const char *homenv = getenv("HOME");
 
+#ifdef HAVE_PWD_H
 	/* When HOME isn't set, or when we're root, get the home directory
 	 * from the password file instead. */
 	if (homenv == NULL || geteuid() == 0) {
@@ -44,6 +44,7 @@ void get_homedir(void)
 	    if (userage != NULL)
 		homenv = userage->pw_dir;
 	}
+#endif
 
 	/* Only set homedir if some home directory could be determined,
 	 * otherwise keep homedir NULL. */
@@ -52,35 +53,57 @@ void get_homedir(void)
     }
 }
 
+/* Return the filename part of the given path. */
+const char *tail(const char *path)
+{
+    const char *slash = strrchr(path, '/');
+
+    if (slash == NULL)
+	return path;
+    else
+	return ++slash;
+}
+
+/* Return a copy of the two given strings, welded together. */
+char *concatenate(const char *path, const char *name)
+{
+    size_t pathlen = strlen(path);
+    char *joined = charalloc(pathlen + strlen(name) + 1);
+
+    strcpy(joined, path);
+    strcpy(joined + pathlen, name);
+
+    return joined;
+}
+
 #ifdef ENABLE_LINENUMBERS
 /* Return the number of digits that the given integer n takes up. */
 int digits(ssize_t n)
 {
     if (n < 100000) {
-        if (n < 1000) {
-            if (n < 100)
-                return 2;
-            else
-                return 3;
-        } else {
-            if (n < 10000)
-                return 4;
-            else
-                return 5;
-        }
+	if (n < 1000) {
+	    if (n < 100)
+		return 2;
+	    else
+		return 3;
+	} else {
+	    if (n < 10000)
+		return 4;
+	    else
+		return 5;
+	}
     } else {
-        if (n < 10000000) {
-            if (n < 1000000)
-                return 6;
-            else
-                return 7;
-        }
-        else {
-            if (n < 100000000)
-                return 8;
-            else
-                return 9;
-        }
+	if (n < 10000000) {
+	    if (n < 1000000)
+		return 6;
+	    else
+		return 7;
+	} else {
+	    if (n < 100000000)
+		return 8;
+	    else
+		return 9;
+	}
     }
 }
 #endif
@@ -114,7 +137,7 @@ bool parse_line_column(const char *str, ssize_t *line, ssize_t *column)
     const char *comma;
 
     while (*str == ' ')
-       str++;
+	str++;
 
     comma = strpbrk(str, "m,. /;");
 
@@ -170,94 +193,6 @@ void sunder(char *str)
     }
 }
 
-/* These functions, ngetline() (originally getline()) and ngetdelim()
- * (originally getdelim()), were adapted from GNU mailutils 0.5
- * (mailbox/getline.c).  Here is the notice from that file, after
- * converting to the GPL via LGPL clause 3, and with the Free Software
- * Foundation's address and the copyright years updated:
- *
- * GNU Mailutils -- a suite of utilities for electronic mail
- * Copyright (C) 1999, 2000, 2001, 2002, 2004, 2005, 2006, 2007
- * Free Software Foundation, Inc.
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 3 of the
- * License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
- * 02110-1301, USA. */
-
-#ifndef DISABLE_NANORC
-
-#ifndef HAVE_GETDELIM
-/* This function is equivalent to getdelim(). */
-ssize_t ngetdelim(char **lineptr, size_t *n, int delim, FILE *stream)
-{
-    size_t indx = 0;
-    int c;
-
-    /* Sanity checks. */
-    if (lineptr == NULL || n == NULL || stream == NULL ||
-	fileno(stream) == -1) {
-	errno = EINVAL;
-	return -1;
-    }
-
-    /* Allocate the line the first time. */
-    if (*lineptr == NULL) {
-	*n = MAX_BUF_SIZE;
-	*lineptr = charalloc(*n);
-    }
-
-    while ((c = getc(stream)) != EOF) {
-	/* Check if more memory is needed. */
-	if (indx >= *n) {
-	    *n += MAX_BUF_SIZE;
-	    *lineptr = charealloc(*lineptr, *n);
-	}
-
-	/* Put the result in the line. */
-	(*lineptr)[indx++] = (char)c;
-
-	/* Bail out. */
-	if (c == delim)
-	    break;
-    }
-
-    /* Make room for the null character. */
-    if (indx >= *n) {
-	*n += MAX_BUF_SIZE;
-	*lineptr = charealloc(*lineptr, *n);
-    }
-
-    /* Null-terminate the buffer. */
-    null_at(lineptr, indx++);
-    *n = indx;
-
-    /* The last line may not have the delimiter.  We have to return what
-     * we got, and the error will be seen on the next iteration. */
-    return (c == EOF && (indx - 1) == 0) ? -1 : indx - 1;
-}
-#endif
-
-#ifndef HAVE_GETLINE
-/* This function is equivalent to getline(). */
-ssize_t ngetline(char **lineptr, size_t *n, FILE *stream)
-{
-    return getdelim(lineptr, n, '\n', stream);
-}
-#endif
-#endif /* !DISABLE_NANORC */
-
-#ifdef HAVE_REGEX_H
 /* Fix the regex if we're on platforms which require an adjustment
  * from GNU-style to BSD-style word boundaries. */
 const char *fixbounds(const char *r)
@@ -267,10 +202,6 @@ const char *fixbounds(const char *r)
     char *r2 = charalloc(strlen(r) * 5);
     char *r3;
 
-#ifdef DEBUG
-    fprintf(stderr, "fixbounds(): Start string = \"%s\"\n", r);
-#endif
-
     for (i = 0; i < strlen(r); i++) {
 	if (r[i] != '\0' && r[i] == '\\' && (r[i + 1] == '>' || r[i + 1] == '<')) {
 	    strcpy(&r2[j], "[[:");
@@ -282,25 +213,23 @@ const char *fixbounds(const char *r)
 	    r2[j] = r[i];
 	j++;
     }
+
     r2[j] = '\0';
     r3 = mallocstrcpy(NULL, r2);
     free(r2);
-#ifdef DEBUG
-    fprintf(stderr, "fixbounds(): Ending string = \"%s\"\n", r3);
-#endif
+
     return (const char *) r3;
 #endif /* !GNU_WORDBOUNDS */
 
     return r;
 }
-#endif /* HAVE_REGEX_H */
 
-#ifndef DISABLE_SPELLER
+#ifdef ENABLE_SPELLER
 /* Is the word starting at the given position in buf and of the given length
  * a separate word?  That is: is it not part of a longer word?*/
 bool is_separate_word(size_t position, size_t length, const char *buf)
 {
-    char before[mb_cur_max()], after[mb_cur_max()];
+    char before[MAXCHARLEN], after[MAXCHARLEN];
     size_t word_end = position + length;
 
     /* Get the characters before and after the word, if any. */
@@ -313,7 +242,7 @@ bool is_separate_word(size_t position, size_t length, const char *buf)
     return ((position == 0 || !is_alpha_mbchar(before)) &&
 		(buf[word_end] == '\0' || !is_alpha_mbchar(after)));
 }
-#endif /* !DISABLE_SPELLER */
+#endif /* ENABLE_SPELLER */
 
 /* Return the position of the needle in the haystack, or NULL if not found.
  * When searching backwards, we will find the last match that starts no later
@@ -323,7 +252,13 @@ bool is_separate_word(size_t position, size_t length, const char *buf)
 const char *strstrwrapper(const char *haystack, const char *needle,
 	const char *start)
 {
-#ifdef HAVE_REGEX_H
+    if (*needle == '\0') {
+#ifndef NANO_TINY
+	statusline(ALERT, "Searching for nothing -- please report a bug");
+#endif
+	return (char *)start;
+    }
+
     if (ISSET(USE_REGEXP)) {
 	if (ISSET(BACKWARDS_SEARCH)) {
 	    size_t last_find, ceiling, far_end;
@@ -362,7 +297,7 @@ const char *strstrwrapper(const char *haystack, const char *needle,
 	    regmatches[0].rm_eo = far_end;
 	    if (regexec(&search_regexp, haystack, 10, regmatches,
 					REG_STARTEND) != 0)
-		statusline(ALERT, "BAD: failed to refind the match!");
+		return NULL;
 
 	    return haystack + regmatches[0].rm_so;
 	}
@@ -376,7 +311,6 @@ const char *strstrwrapper(const char *haystack, const char *needle,
 	else
 	    return haystack + regmatches[0].rm_so;
     }
-#endif /* HAVE_REGEX_H */
     if (ISSET(CASE_SENSITIVE)) {
 	if (ISSET(BACKWARDS_SEARCH))
 	    return revstrstr(haystack, needle, start);
@@ -454,13 +388,14 @@ char *free_and_assign(char *dest, char *src)
     return src;
 }
 
-/* nano scrolls horizontally within a line in chunks.  Return the column
- * number of the first character displayed in the edit window when the
+/* When not in softwrap mode, nano scrolls horizontally within a line in
+ * chunks (a bit smaller than the chunks used in softwrapping).  Return the
+ * column number of the first character displayed in the edit window when the
  * cursor is at the given column.  Note that (0 <= column -
  * get_page_start(column) < COLS). */
 size_t get_page_start(size_t column)
 {
-    if (column == 0 || column < editwincols - 1)
+    if (column < editwincols - 1 || ISSET(SOFTWRAP) || column == 0)
 	return 0;
     else if (editwincols > 8)
 	return column - 7 - (column - 7) % (editwincols - 8);
@@ -479,10 +414,10 @@ size_t xplustabs(void)
  * not overshoot the given column. */
 size_t actual_x(const char *text, size_t column)
 {
-    size_t index = 0;
-	/* The index in text, returned. */
+    const char *start = text;
+	/* From where we start walking through the text. */
     size_t width = 0;
-	/* The screen display width to text[index], in columns. */
+	/* The current accumulated span, in columns. */
 
     while (*text != '\0') {
 	int charlen = parse_mbchar(text, NULL, &width);
@@ -490,11 +425,10 @@ size_t actual_x(const char *text, size_t column)
 	if (width > column)
 	    break;
 
-	index += charlen;
 	text += charlen;
     }
 
-    return index;
+    return (text - start);
 }
 
 /* A strnlen() with tabs and multicolumn characters factored in:
@@ -520,11 +454,15 @@ size_t strnlenpt(const char *text, size_t maxlen)
     return width;
 }
 
-/* A strlen() with tabs and multicolumn characters factored in:
- * how many columns wide is text? */
+/* Return the number of columns that the given text occupies. */
 size_t strlenpt(const char *text)
 {
-    return strnlenpt(text, (size_t)-1);
+    size_t span = 0;
+
+    while (*text != '\0')
+	text += parse_mbchar(text, NULL, &span);
+
+    return span;
 }
 
 /* Append a new magicline to filebot. */
@@ -536,7 +474,7 @@ void new_magicline(void)
     openfile->totsize++;
 }
 
-#ifndef NANO_TINY
+#if !defined(NANO_TINY) || defined(ENABLE_HELP)
 /* Remove the magicline from filebot, if there is one and it isn't the
  * only line in the file.  Assume that edittop and current are not at
  * filebot. */
@@ -550,7 +488,9 @@ void remove_magicline(void)
 	openfile->totsize--;
     }
 }
+#endif
 
+#ifndef NANO_TINY
 /* Set top_x and bot_x to the top and bottom x-coordinates of the mark,
  * respectively, based on the locations of top and bot.  If
  * right_side_up isn't NULL, set it to TRUE if the mark begins with
@@ -591,8 +531,7 @@ filestruct *fsfromline(ssize_t lineno)
 	    f = f->next;
 
     if (f->lineno != lineno) {
-	statusline(ALERT, _("Internal error: can't match line %ld.  "
-			"Please save your work."), (long)lineno);
+	statusline(ALERT, "Gone undo line -- please report a bug");
 	return NULL;
     }
 
@@ -619,7 +558,7 @@ size_t get_totsize(const filestruct *begin, const filestruct *end)
 }
 
 #ifdef DEBUG
-/* Dump the filestruct inptr to stderr. */
+/* Dump the given buffer to stderr. */
 void dump_filestruct(const filestruct *inptr)
 {
     if (inptr == openfile->fileage)
@@ -630,19 +569,18 @@ void dump_filestruct(const filestruct *inptr)
 	fprintf(stderr, "Dumping a buffer to stderr...\n");
 
     while (inptr != NULL) {
-	fprintf(stderr, "(%ld) %s\n", (long)inptr->lineno, inptr->data);
+	fprintf(stderr, "(%zd) %s\n", inptr->lineno, inptr->data);
 	inptr = inptr->next;
     }
 }
 
-/* Dump the current buffer's filestruct to stderr in reverse. */
+/* Dump the current buffer to stderr in reverse. */
 void dump_filestruct_reverse(void)
 {
     const filestruct *fileptr = openfile->filebot;
 
     while (fileptr != NULL) {
-	fprintf(stderr, "(%ld) %s\n", (long)fileptr->lineno,
-		fileptr->data);
+	fprintf(stderr, "(%zd) %s\n", fileptr->lineno, fileptr->data);
 	fileptr = fileptr->prev;
     }
 }
